#! /usr/bin/perl -w
#
# check_mssql: simple plugin using sqsh to check an MS SQL server
#
# Copyright  2002 Jonathan Rozes <jrozes@vinton.com>
#

require 5.004;
use Getopt::Long;
use strict;
use vars qw($opt_V $opt_h $opt_H $opt_f $opt_w $opt_c $PROGNAME);
use lib "/var/local/nagios/libexec"  ;
use utils qw($TIMEOUT %ERRORS &print_revision &support);

$PROGNAME="check_mssql";

sub print_help ();
sub print_usage ();

$ENV{'PATH'}='';
$ENV{'BASH_ENV'}='';
$ENV{'ENV'}='';

Getopt::Long::Configure('bundling');
GetOptions
	("V"   => \$opt_V, "version"    => \$opt_V,
	 "h"   => \$opt_h, "help"       => \$opt_h,
	 "f=s" => \$opt_f, "file=s"     => \$opt_f,
	 "w=s" => \$opt_w, "warn=s"     => \$opt_w,
	 "c=s" => \$opt_c, "crit=s"     => \$opt_c,
	 "H=s" => \$opt_H, "hostname=s" => \$opt_H);

if ($opt_V) {
	print_revision($PROGNAME,'$Revision: 1.0 $ ');
	exit $ERRORS{'OK'};
}

if ($opt_h) {
	print_help();
	exit $ERRORS{'OK'};
}

$opt_H = shift unless ($opt_H);
my $host = $1 if ($opt_H && $opt_H =~ m/^([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|[a-zA-Z][-a-zA-Z0-9]+(\.[a-zA-Z][-a-zA-Z0-9]+)*)$/);
unless ($host) {
	print "No target host specified\n";
	print_usage();
	exit $ERRORS{'UNKNOWN'};
}

my $file = $opt_f if ($opt_f);
unless ($file) {
	print "No command file specified\n";
	print_usage();
	exit $ERRORS{'UNKNOWN'};
}

$opt_w = shift unless ($opt_w);
my $warn = $1 if ($opt_w && $opt_w =~ m/^([0-9]+\.*[0-9]*)$/);
$warn = '5' unless ($warn);

$opt_c = shift unless ($opt_c);
my $crit = $1 if ($opt_c && $opt_c =~ m/^([0-9]+\.*[0-9]*)$/);
$crit = '10' unless ($crit);

if ($crit < $warn) {
	print "Critical threshold $crit must be greater than warning threshold $warn\n";
	exit $ERRORS{'UNKNOWN'};
}

unless ( -r $file ) {
	print "File $file does not exist\n";
	print_usage();
	exit $ERRORS{'UNKNOWN'};
}

my $sqsh = '/usr/local/bin/sqsh';
unless ( -x $sqsh ) {
	print "Unable to execute $sqsh\n";
	exit $ERRORS{'UNKNOWN'};
}

$SIG{'ALRM'} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print ("SQL ERROR - No response from SQL server (alarm)\n");
	exit $ERRORS{'UNKNOWN'};
};
alarm($TIMEOUT);

my @out = `$sqsh -b -p -S $host -i $file 2>&1`;
my $ret = $? >> 8;
my ($maxtot,$stat);
foreach (@out) {
	if (/Total = ([0-9]+\.*[0-9]*)/) {
		my $s = (split /:\s+/,$_)[1];
		chomp $s;
		if (! defined $maxtot || $1 >= $maxtot) {
			$maxtot = $1;
			$stat = $s;
		}
	}
}

if ($ret == 255) {
	print "SQL CRITICAL - $out[0]\n";
	exit $ERRORS{'CRITICAL'};
} elsif ($ret) {
	print "SQL CRITICAL - $ret statements failed during execution\n";
	exit $ERRORS{'CRITICAL'};
} elsif (! defined $maxtot) {
	print "SQL WARNING - no timing data returned\n";
	exit $ERRORS{'WARNING'};
} elsif ($maxtot > $crit) {
	print "SQL CRITICAL - query time $maxtot > critical threshold $crit\n";
	exit $ERRORS{'CRITICAL'};
} elsif ($maxtot > $warn) {
	print "SQL WARNING - query time $maxtot > warning threshold $warn\n";
	exit $ERRORS{'WARNING'};
} else {
	print "SQL ok - $stat\n";
	exit $ERRORS{'OK'};
}

sub print_usage () {
	print "Usage: $PROGNAME -H <host> -c <file> [ -w <sec> ] [ -c <sec> ]\n";
}

sub print_help () {
	print_revision($PROGNAME,'$Revision: 1.0 $');
	print "Copyright  2002 Jonathan Rozes\n";
	print "\n";
	print_usage();
	print "\n";
	print "-f <file> = Read SQL commands from <file>\n";
	print "-w <sec>  = Return warning if query takes longer than <sec> seconds\n";
	print "            Default is 5 seconds\n";
	print "-c <sec>  = Return critical if query takes longer than <sec> seconds\n";
	print "            Default is 10 seconds\n";
	support();
}
